@@ -10,6 +10,9 @@ type JJConfig struct {
 	Revsets       struct {
 		Log string `toml:"log"`
 	} `toml:"revsets"`
+	Templates struct {
+		Log string `toml:"log"`
+	} `toml:"templates"`
 }
 
 func (c *JJConfig) GetApplicableColors() map[string]Color {
@@ -23,6 +26,7 @@ func (c *JJConfig) GetApplicableColors() map[string]Color {
 		"diff copied",
 		"diff modified",
 		"diff removed",
+		"bookmark",
 		"change_id",
 		"commit_id",
 		"conflict",
@@ -41,7 +45,6 @@ func parseConfig(configContent string) (*JJConfig, error) {
 	if err != nil {
 		return nil, err
 	}
-
 	return &config, nil
 }
 
