@@ -34,24 +34,26 @@ func Convert(m KeyMappings[keys]) KeyMappings[key.Binding] {
 		Absorb:            key.NewBinding(key.WithKeys(m.Absorb...), key.WithHelp(JoinKeys(m.Absorb), "absorb")),
 		Split:             key.NewBinding(key.WithKeys(m.Split...), key.WithHelp(JoinKeys(m.Split), "split")),
 		SplitParallel:     key.NewBinding(key.WithKeys(m.SplitParallel...), key.WithHelp(JoinKeys(m.SplitParallel), "split (parallel)")),
-		Help:              key.NewBinding(key.WithKeys(m.Help...), key.WithHelp(JoinKeys(m.Help), "help")),
+		ExpandStatus:      key.NewBinding(key.WithKeys(m.ExpandStatus...), key.WithHelp(JoinKeys(m.ExpandStatus), "expand status")),
 		Evolog: evologModeKeys[key.Binding]{
 			Mode:    key.NewBinding(key.WithKeys(m.Evolog.Mode...), key.WithHelp(JoinKeys(m.Evolog.Mode), "evolog")),
 			Diff:    key.NewBinding(key.WithKeys(m.Evolog.Diff...), key.WithHelp(JoinKeys(m.Evolog.Diff), "diff")),
 			Restore: key.NewBinding(key.WithKeys(m.Evolog.Restore...), key.WithHelp(JoinKeys(m.Evolog.Restore), "restore")),
 		},
-		Revset:           key.NewBinding(key.WithKeys(m.Revset...), key.WithHelp(JoinKeys(m.Revset), "revset")),
-		AceJump:          key.NewBinding(key.WithKeys(m.AceJump...), key.WithHelp(JoinKeys(m.AceJump), "ace jump")),
-		QuickSearch:      key.NewBinding(key.WithKeys(m.QuickSearch...), key.WithHelp(JoinKeys(m.QuickSearch), "quick search")),
-		QuickSearchCycle: key.NewBinding(key.WithKeys(m.QuickSearchCycle...), key.WithHelp(JoinKeys(m.QuickSearchCycle), "locate next match")),
-		CustomCommands:   key.NewBinding(key.WithKeys(m.CustomCommands...), key.WithHelp(JoinKeys(m.CustomCommands), "custom commands menu")),
-		Leader:           key.NewBinding(key.WithKeys(m.Leader...), key.WithHelp(JoinKeys(m.Leader), "leader")),
-		Suspend:          key.NewBinding(key.WithKeys(m.Suspend...), key.WithHelp(JoinKeys(m.Suspend), "suspend")),
-		SetParents:       key.NewBinding(key.WithKeys(m.SetParents...), key.WithHelp(JoinKeys(m.SetParents), "set parents")),
-		ExecJJ:           key.NewBinding(key.WithKeys(m.ExecJJ...), key.WithHelp(JoinKeys(m.ExecJJ), "interactive jj")),
-		ExecShell:        key.NewBinding(key.WithKeys(m.ExecShell...), key.WithHelp(JoinKeys(m.ExecShell), "interactive shell command")),
+		Revset:          key.NewBinding(key.WithKeys(m.Revset...), key.WithHelp(JoinKeys(m.Revset), "revset")),
+		AceJump:         key.NewBinding(key.WithKeys(m.AceJump...), key.WithHelp(JoinKeys(m.AceJump), "ace jump")),
+		QuickSearch:     key.NewBinding(key.WithKeys(m.QuickSearch...), key.WithHelp(JoinKeys(m.QuickSearch), "quick search")),
+		QuickSearchNext: key.NewBinding(key.WithKeys(m.QuickSearchNext...), key.WithHelp(JoinKeys(m.QuickSearchNext), "locate next match")),
+		QuickSearchPrev: key.NewBinding(key.WithKeys(m.QuickSearchPrev...), key.WithHelp(JoinKeys(m.QuickSearchPrev), "locate previous match")),
+		CustomCommands:  key.NewBinding(key.WithKeys(m.CustomCommands...), key.WithHelp(JoinKeys(m.CustomCommands), "custom commands menu")),
+		Leader:          key.NewBinding(key.WithKeys(m.Leader...), key.WithHelp(JoinKeys(m.Leader), "leader")),
+		Suspend:         key.NewBinding(key.WithKeys(m.Suspend...), key.WithHelp(JoinKeys(m.Suspend), "suspend")),
+		SetParents:      key.NewBinding(key.WithKeys(m.SetParents...), key.WithHelp(JoinKeys(m.SetParents), "set parents")),
+		ExecJJ:          key.NewBinding(key.WithKeys(m.ExecJJ...), key.WithHelp(JoinKeys(m.ExecJJ), "interactive jj")),
+		ExecShell:       key.NewBinding(key.WithKeys(m.ExecShell...), key.WithHelp(JoinKeys(m.ExecShell), "interactive shell command")),
 		Revert: revertModeKeys[key.Binding]{
 			Mode:   key.NewBinding(key.WithKeys(m.Revert.Mode...), key.WithHelp(JoinKeys(m.Revert.Mode), "revert")),
+			Target: key.NewBinding(key.WithKeys(m.Revert.Target...), key.WithHelp(JoinKeys(m.Revert.Target), "target")),
 			After:  key.NewBinding(key.WithKeys(m.Revert.After...), key.WithHelp(JoinKeys(m.Revert.After), "insert after")),
 			Before: key.NewBinding(key.WithKeys(m.Revert.Before...), key.WithHelp(JoinKeys(m.Revert.Before), "insert before")),
 			Onto:   key.NewBinding(key.WithKeys(m.Revert.Onto...), key.WithHelp(JoinKeys(m.Revert.Onto), "onto")),
@@ -62,6 +64,7 @@ func Convert(m KeyMappings[keys]) KeyMappings[key.Binding] {
 			Revision:    key.NewBinding(key.WithKeys(m.Rebase.Revision...), key.WithHelp(JoinKeys(m.Rebase.Revision), "revision")),
 			Source:      key.NewBinding(key.WithKeys(m.Rebase.Source...), key.WithHelp(JoinKeys(m.Rebase.Source), "source")),
 			Branch:      key.NewBinding(key.WithKeys(m.Rebase.Branch...), key.WithHelp(JoinKeys(m.Rebase.Branch), "branch")),
+			Target:      key.NewBinding(key.WithKeys(m.Rebase.Target...), key.WithHelp(JoinKeys(m.Rebase.Target), "target")),
 			After:       key.NewBinding(key.WithKeys(m.Rebase.After...), key.WithHelp(JoinKeys(m.Rebase.After), "insert after")),
 			Before:      key.NewBinding(key.WithKeys(m.Rebase.Before...), key.WithHelp(JoinKeys(m.Rebase.Before), "insert before")),
 			Onto:        key.NewBinding(key.WithKeys(m.Rebase.Onto...), key.WithHelp(JoinKeys(m.Rebase.Onto), "onto")),
@@ -70,12 +73,14 @@ func Convert(m KeyMappings[keys]) KeyMappings[key.Binding] {
 		},
 		Duplicate: duplicateModeKeys[key.Binding]{
 			Mode:   key.NewBinding(key.WithKeys(m.Duplicate.Mode...), key.WithHelp(JoinKeys(m.Duplicate.Mode), "duplicate")),
+			Target: key.NewBinding(key.WithKeys(m.Duplicate.Target...), key.WithHelp(JoinKeys(m.Duplicate.Target), "target")),
 			After:  key.NewBinding(key.WithKeys(m.Duplicate.After...), key.WithHelp(JoinKeys(m.Duplicate.After), "duplicate after")),
 			Before: key.NewBinding(key.WithKeys(m.Duplicate.Before...), key.WithHelp(JoinKeys(m.Duplicate.Before), "duplicate before")),
 			Onto:   key.NewBinding(key.WithKeys(m.Duplicate.Onto...), key.WithHelp(JoinKeys(m.Duplicate.Onto), "duplicate onto")),
 		},
 		Squash: squashModeKeys[key.Binding]{
 			Mode:                  key.NewBinding(key.WithKeys(m.Squash.Mode...), key.WithHelp(JoinKeys(m.Squash.Mode), "squash")),
+			Target:                key.NewBinding(key.WithKeys(m.Squash.Target...), key.WithHelp(JoinKeys(m.Squash.Target), "target")),
 			KeepEmptied:           key.NewBinding(key.WithKeys(m.Squash.KeepEmptied...), key.WithHelp(JoinKeys(m.Squash.KeepEmptied), "keep emptied commits")),
 			UseDestinationMessage: key.NewBinding(key.WithKeys(m.Squash.UseDestinationMessage...), key.WithHelp(JoinKeys(m.Squash.UseDestinationMessage), "use destination message")),
 			Interactive:           key.NewBinding(key.WithKeys(m.Squash.Interactive...), key.WithHelp(JoinKeys(m.Squash.Interactive), "interactive")),
@@ -89,7 +94,7 @@ func Convert(m KeyMappings[keys]) KeyMappings[key.Binding] {
 			Restore:               key.NewBinding(key.WithKeys(m.Details.Restore...), key.WithHelp(JoinKeys(m.Details.Restore), "restore")),
 			Absorb:                key.NewBinding(key.WithKeys(m.Details.Absorb...), key.WithHelp(JoinKeys(m.Details.Absorb), "absorb")),
 			Diff:                  key.NewBinding(key.WithKeys(m.Details.Diff...), key.WithHelp(JoinKeys(m.Details.Diff), "diff")),
-			ToggleSelect:          key.NewBinding(key.WithKeys(m.Details.ToggleSelect...), key.WithHelp(JoinKeys(m.Details.ToggleSelect), "details toggle select")),
+			ToggleSelect:          key.NewBinding(key.WithKeys(m.Details.ToggleSelect...), key.WithHelp(JoinKeys(m.Details.ToggleSelect), "toggle select")),
 			RevisionsChangingFile: key.NewBinding(key.WithKeys(m.Details.RevisionsChangingFile...), key.WithHelp(JoinKeys(m.Details.RevisionsChangingFile), "show revisions changing file")),
 		},
 		Bookmark: bookmarkModeKeys[key.Binding]{
@@ -149,6 +154,10 @@ func JoinKeys(keys []string) string {
 			k = "↑"
 		case "down":
 			k = "↓"
+		case "left":
+			k = "←"
+		case "right":
+			k = "→"
 		case " ":
 			k = "space"
 		}
@@ -177,7 +186,7 @@ type KeyMappings[T any] struct {
 	Abandon           T                         `toml:"abandon"`
 	Diff              T                         `toml:"diff"`
 	Quit              T                         `toml:"quit"`
-	Help              T                         `toml:"help"`
+	ExpandStatus      T                         `toml:"expand_status"`
 	Describe          T                         `toml:"describe"`
 	Edit              T                         `toml:"edit"`
 	ForceEdit         T                         `toml:"force_edit"`
@@ -192,7 +201,8 @@ type KeyMappings[T any] struct {
 	ExecShell         T                         `toml:"exec_shell"`
 	AceJump           T                         `toml:"ace_jump"`
 	QuickSearch       T                         `toml:"quick_search"`
-	QuickSearchCycle  T                         `toml:"quick_search_cycle"`
+	QuickSearchNext   T                         `toml:"quick_search_cycle"`
+	QuickSearchPrev   T                         `toml:"quick_search_cycle_back"`
 	CustomCommands    T                         `toml:"custom_commands"`
 	Leader            T                         `toml:"leader"`
 	Suspend           T                         `toml:"suspend"`
@@ -223,6 +233,7 @@ type bookmarkModeKeys[T any] struct {
 
 type squashModeKeys[T any] struct {
 	Mode                  T `toml:"mode"`
+	Target                T `toml:"target"`
 	KeepEmptied           T `toml:"keep_emptied"`
 	UseDestinationMessage T `toml:"use_destination_message"`
 	Interactive           T `toml:"interactive"`
@@ -230,6 +241,7 @@ type squashModeKeys[T any] struct {
 
 type revertModeKeys[T any] struct {
 	Mode   T `toml:"mode"`
+	Target T `toml:"target"`
 	After  T `toml:"after"`
 	Before T `toml:"before"`
 	Onto   T `toml:"onto"`
@@ -241,6 +253,7 @@ type rebaseModeKeys[T any] struct {
 	Revision    T `toml:"revision"`
 	Source      T `toml:"source"`
 	Branch      T `toml:"branch"`
+	Target      T `toml:"target"`
 	After       T `toml:"after"`
 	Before      T `toml:"before"`
 	Onto        T `toml:"onto"`
@@ -250,6 +263,7 @@ type rebaseModeKeys[T any] struct {
 
 type duplicateModeKeys[T any] struct {
 	Mode   T `toml:"mode"`
+	Target T `toml:"target"`
 	After  T `toml:"after"`
 	Before T `toml:"before"`
 	Onto   T `toml:"onto"`
