@@ -30,6 +30,10 @@ const (
 )
 
 type CommandArgs []string
+type CommandWithStdin struct {
+	Args  CommandArgs
+	Input string
+}
 
 func ConfigListAll() CommandArgs {
 	return []string{"config", "list", "--color", "never", "--include-defaults", "--ignore-working-copy"}
@@ -78,21 +82,14 @@ func DiffEdit(changeId string) CommandArgs {
 	return []string{"diffedit", "-r", changeId}
 }
 
-func Split(revision string, files []string, parallel bool) CommandArgs {
+func Split(revision string, files []string, parallel bool, interactive bool) CommandArgs {
 	args := []string{"split", "-r", revision}
 	if parallel {
 		args = append(args, "--parallel")
 	}
-	var escapedFiles []string
-	for _, file := range files {
-		escapedFiles = append(escapedFiles, EscapeFileName(file))
+	if interactive {
+		args = append(args, "--interactive")
 	}
-	args = append(args, escapedFiles...)
-	return args
-}
-
-func SquashFiles(from string, into string, files []string) CommandArgs {
-	args := []string{"squash", "--from", from, "--into", into, "--use-destination-message"}
 	var escapedFiles []string
 	for _, file := range files {
 		escapedFiles = append(escapedFiles, EscapeFileName(file))
@@ -102,13 +99,16 @@ func SquashFiles(from string, into string, files []string) CommandArgs {
 }
 
 func Describe(revisions SelectedRevisions) CommandArgs {
-	args := []string{"describe", "--edit"}
+	args := []string{"describe", "--editor"}
 	args = append(args, revisions.AsArgs()...)
 	return args
 }
 
-func SetDescription(revision string, description string) CommandArgs {
-	return []string{"describe", "-r", revision, "-m", description}
+func SetDescription(revision string, description string) CommandWithStdin {
+	return CommandWithStdin{
+		Args:  []string{"describe", "-r", revision, "--stdin"},
+		Input: description,
+	}
 }
 
 func GetDescription(revision string) CommandArgs {
@@ -135,8 +135,11 @@ func Diff(revision string, fileName string, extraArgs ...string) CommandArgs {
 	return args
 }
 
-func Restore(revision string, files []string) CommandArgs {
+func Restore(revision string, files []string, interactive bool) CommandArgs {
 	args := []string{"restore", "-c", revision}
+	if interactive {
+		args = append(args, "--interactive")
+	}
 	var escapedFiles []string
 	for _, file := range files {
 		escapedFiles = append(escapedFiles, EscapeFileName(file))
@@ -145,14 +148,6 @@ func Restore(revision string, files []string) CommandArgs {
 	return args
 }
 
-func RestoreInteractive(revision string, file string) CommandArgs {
-	args := []string{"restore", "-c", revision, "--interactive"}
-	if file != "" {
-		args = append(args, EscapeFileName(file))
-	}
-	return args
-}
-
 func RestoreEvolog(from string, into string) CommandArgs {
 	args := []string{"restore", "--from", from, "--into", into, "--restore-descendants"}
 	return args
@@ -195,12 +190,20 @@ func BookmarkForget(name string) CommandArgs {
 	return []string{"bookmark", "forget", name}
 }
 
-func BookmarkTrack(name string) CommandArgs {
-	return []string{"bookmark", "track", name}
+func BookmarkTrack(name string, remote string) CommandArgs {
+	args := []string{"bookmark", "track", name}
+	if remote != "" {
+		args = append(args, "--remote", remote)
+	}
+	return args
 }
 
-func BookmarkUntrack(name string) CommandArgs {
-	return []string{"bookmark", "untrack", name}
+func BookmarkUntrack(name string, remote string) CommandArgs {
+	args := []string{"bookmark", "untrack", name}
+	if remote != "" {
+		args = append(args, "--remote", remote)
+	}
+	return args
 }
 
 func Squash(from SelectedRevisions, destination string, files []string, keepEmptied bool, useDestinationMessage bool, interactive bool, ignoreImmutable bool) CommandArgs {
@@ -246,6 +249,10 @@ func BookmarkListAll() CommandArgs {
 	return []string{"bookmark", "list", "-a", "--template", allBookmarkTemplate, "--color", "never", "--ignore-working-copy"}
 }
 
+func TagList() CommandArgs {
+	return []string{"tag", "list", "--template", "name ++ '\n'", "--color", "never", "--ignore-working-copy"}
+}
+
 func GitFetch(flags ...string) CommandArgs {
 	args := []string{"git", "fetch"}
 	if flags != nil {
@@ -266,14 +273,6 @@ func GitRemoteList() CommandArgs {
 	return []string{"git", "remote", "list"}
 }
 
-func Show(revision string, extraArgs ...string) CommandArgs {
-	args := []string{"show", "-r", revision, "--color", "always", "--ignore-working-copy"}
-	if extraArgs != nil {
-		args = append(args, extraArgs...)
-	}
-	return args
-}
-
 func Rebase(from SelectedRevisions, to string, source string, target string, skipEmptied bool, ignoreImmutable bool) CommandArgs {
 	args := []string{"rebase"}
 	args = append(args, from.AsPrefixedArgs(source)...)
