@@ -1,9 +1,9 @@
 package common
 
 type ViewRange struct {
-	*Sizeable
+	Width         int
+	Height        int
 	Start         int
-	End           int
 	FirstRowIndex int
 	LastRowIndex  int
 }
