@@ -8,10 +8,13 @@ import (
 	"log"
 	"os"
 	"os/exec"
+	"runtime"
 	"slices"
+	"strings"
 	"sync"
 
-	"github.com/charmbracelet/bubbletea"
+	tea "github.com/charmbracelet/bubbletea"
+	"github.com/idursun/jjui/internal/askpass"
 	"github.com/idursun/jjui/internal/ui/common"
 )
 
@@ -19,11 +22,13 @@ type CommandRunner interface {
 	RunCommandImmediate(args []string) ([]byte, error)
 	RunCommandStreaming(ctx context.Context, args []string) (*StreamingCommand, error)
 	RunCommand(args []string, continuations ...tea.Cmd) tea.Cmd
+	RunCommandWithInput(args []string, input string, continuations ...tea.Cmd) tea.Cmd
 	RunInteractiveCommand(args []string, continuation tea.Cmd) tea.Cmd
 }
 
 type MainCommandRunner struct {
 	Location string
+	Askpass  *askpass.Server
 }
 
 func (a *MainCommandRunner) RunCommandImmediate(args []string) ([]byte, error) {
@@ -62,22 +67,50 @@ func (a *MainCommandRunner) RunCommandStreaming(ctx context.Context, args []stri
 	}, nil
 }
 
-func (a *MainCommandRunner) RunCommand(args []string, continuations ...tea.Cmd) tea.Cmd {
+func (a *MainCommandRunner) runCommandWithInput(args []string, input *string, continuations []tea.Cmd) tea.Cmd {
 	commands := make([]tea.Cmd, 0)
 	commands = append(commands,
 		func() tea.Msg {
+			started, cancel, env := a.Askpass.NewSubprocess(strings.Join(args, " "))
+			defer cancel()
 			if !slices.Contains(args, "--color") {
 				args = append([]string{"--color", "always"}, args...)
 			}
 			c := exec.Command("jj", args...)
 			c.Dir = a.Location
+			c.Env = append(os.Environ(), env...)
+
+			if input != nil {
+				stdin, err := c.StdinPipe()
+				if err != nil {
+					return common.CommandCompletedMsg{
+						Err: err,
+					}
+				}
+				go func() {
+					defer stdin.Close()
+					io.WriteString(stdin, *input)
+				}()
+			}
+
 			var output bytes.Buffer
 			c.Stderr = &output
-			_, err := c.Output()
+			if err := c.Start(); err != nil {
+				return common.CommandCompletedMsg{
+					Err: err,
+				}
+			}
+			started(c.Process.Pid)
+
+			err := c.Wait()
 			if err != nil {
 				var exitError *exec.ExitError
 				if errors.As(err, &exitError) {
-					err = errors.New(output.String())
+					msg := output.String()
+					if len(env) == 0 && slices.Contains([]string{"linux", "darwin"}, runtime.GOOS) {
+						msg += "\nHint: enable ssh.hijack_askpass if you expected a password prompt (e.g. ssh passphrase)"
+					}
+					err = errors.New(msg)
 				}
 			}
 			return common.CommandCompletedMsg{
@@ -92,6 +125,14 @@ func (a *MainCommandRunner) RunCommand(args []string, continuations ...tea.Cmd)
 	)
 }
 
+func (a *MainCommandRunner) RunCommandWithInput(args []string, input string, continuations ...tea.Cmd) tea.Cmd {
+	return a.runCommandWithInput(args, &input, continuations)
+}
+
+func (a *MainCommandRunner) RunCommand(args []string, continuations ...tea.Cmd) tea.Cmd {
+	return a.runCommandWithInput(args, nil, continuations)
+}
+
 func (a *MainCommandRunner) RunInteractiveCommand(args []string, continuation tea.Cmd) tea.Cmd {
 	c := exec.Command("jj", args...)
 	errBuffer := &bytes.Buffer{}
