@@ -1,12 +1,14 @@
 package git
 
 import (
+	"fmt"
 	"testing"
 
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/jj"
-	"github.com/idursun/jjui/internal/ui/common"
+	"github.com/idursun/jjui/internal/ui/layout"
+	"github.com/idursun/jjui/internal/ui/render"
 	"github.com/idursun/jjui/test"
 	"github.com/stretchr/testify/assert"
 )
@@ -18,9 +20,8 @@ func Test_Push(t *testing.T) {
 	defer commandRunner.Verify()
 
 	op := NewModel(test.NewTestContext(commandRunner), jj.NewSelectedRevisions())
-	op.SetFrame(cellbuf.Rect(0, 0, 100, 40))
-	op.Parent = common.NewViewNode(100, 40)
 	test.SimulateModel(op, op.Init())
+	_ = test.RenderImmediate(op, 100, 40)
 	test.SimulateModel(op, test.Press(tea.KeyEnter))
 }
 
@@ -31,9 +32,8 @@ func Test_Fetch(t *testing.T) {
 	defer commandRunner.Verify()
 
 	op := NewModel(test.NewTestContext(commandRunner), jj.NewSelectedRevisions())
-	op.SetFrame(cellbuf.Rect(0, 0, 100, 40))
-	op.Parent = common.NewViewNode(100, 40)
 	test.SimulateModel(op, op.Init())
+	_ = test.RenderImmediate(op, 100, 40)
 	test.SimulateModel(op, test.Type("/fetch"))
 	test.SimulateModel(op, test.Press(tea.KeyEnter))
 	test.SimulateModel(op, test.Press(tea.KeyEnter))
@@ -65,12 +65,37 @@ func Test_PushChange(t *testing.T) {
 	defer commandRunner.Verify()
 
 	op := NewModel(test.NewTestContext(commandRunner), jj.NewSelectedRevisions(&jj.Commit{ChangeId: changeId}))
-	op.SetFrame(cellbuf.Rect(0, 0, 100, 40))
-	op.Parent = common.NewViewNode(100, 40)
 	test.SimulateModel(op, op.Init())
+	_ = test.RenderImmediate(op, 100, 40)
 
 	// Filter for the exact item and ensure selection is at index 0
 	test.SimulateModel(op, test.Type("/git push --change"))
 	test.SimulateModel(op, test.Press(tea.KeyDown)) // Ensure first item is selected
 	test.SimulateModel(op, test.Press(tea.KeyEnter))
+	test.SimulateModel(op, test.Press(tea.KeyEnter))
+}
+
+// TestGit_ZIndex_RendersAboveMainContent verifies that the git overlay renders
+// at z-index >= render.ZMenuBorder. This ensures the git operations menu
+// renders above the main revision list content.
+func TestGit_ZIndex_RendersAboveMainContent(t *testing.T) {
+	commandRunner := test.NewTestCommandRunner(t)
+	commandRunner.Expect(jj.GitRemoteList()).SetOutput([]byte("origin"))
+
+	op := NewModel(test.NewTestContext(commandRunner), jj.NewSelectedRevisions())
+	test.SimulateModel(op, op.Init())
+
+	dl := render.NewDisplayContext()
+	box := layout.Box{R: cellbuf.Rect(0, 0, 100, 40)}
+	op.ViewRect(dl, box)
+
+	draws := dl.DrawList()
+	assert.NotEqual(t, 0, len(draws), "Expected git overlay to produce draw operations")
+
+	for i, draw := range draws {
+		msg := fmt.Sprintf("Draw operation %d has z-index %d, expected >= %d. "+
+			"Git overlay must render above main content.",
+			i, draw.Z, render.ZMenuBorder)
+		assert.GreaterOrEqual(t, draw.Z, render.ZMenuBorder, msg)
+	}
 }
