@@ -6,12 +6,15 @@ import (
 	"github.com/charmbracelet/bubbles/textarea"
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/lipgloss"
+	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/config"
 	"github.com/idursun/jjui/internal/jj"
 	"github.com/idursun/jjui/internal/ui/common"
 	"github.com/idursun/jjui/internal/ui/context"
 	"github.com/idursun/jjui/internal/ui/intents"
+	"github.com/idursun/jjui/internal/ui/layout"
 	"github.com/idursun/jjui/internal/ui/operations"
+	"github.com/idursun/jjui/internal/ui/render"
 )
 
 var (
@@ -27,7 +30,6 @@ type stashedDescription struct {
 }
 
 type Operation struct {
-	*common.ViewNode
 	context      *context.MainContext
 	keyMap       config.KeyMappings[key.Binding]
 	input        textarea.Model
@@ -59,7 +61,34 @@ func (o *Operation) Render(commit *jj.Commit, pos operations.RenderPosition) str
 	if pos != operations.RenderOverDescription {
 		return ""
 	}
-	return o.View()
+	return o.viewContent(80)
+}
+
+func (o *Operation) RenderToDisplayContext(dl *render.DisplayContext, commit *jj.Commit, pos operations.RenderPosition, rect cellbuf.Rectangle, screenOffset cellbuf.Position) int {
+	if pos != operations.RenderOverDescription {
+		return 0
+	}
+	width := rect.Dx()
+	height := o.DesiredHeight(commit, pos)
+
+	o.input.SetWidth(width)
+	o.input.SetHeight(height)
+	content := o.input.View()
+
+	drawRect := cellbuf.Rect(rect.Min.X, rect.Min.Y, width, height)
+	dl.AddDraw(drawRect, content, 0)
+	return height
+}
+
+func (o *Operation) DesiredHeight(_ *jj.Commit, pos operations.RenderPosition) int {
+	if pos != operations.RenderOverDescription {
+		return 0
+	}
+	h := lipgloss.Height(o.input.Value())
+	if h <= 0 {
+		h = 1
+	}
+	return h + 1
 }
 
 func (o *Operation) Name() string {
@@ -77,67 +106,85 @@ func (o *Operation) Update(msg tea.Msg) tea.Cmd {
 	case tea.KeyMsg:
 		switch {
 		case key.Matches(msg, o.keyMap.Cancel):
-			unsavedDescription := o.input.Value()
-			if o.originalDesc != unsavedDescription {
-				stashed = &stashedDescription{
-					revision:    o.revision,
-					description: unsavedDescription,
-				}
-				return tea.Batch(common.Close, intents.Invoke(intents.AddMessage{Text: "Unsaved description is stashed. Edit again to restore."}))
-			}
-			return common.Close
+			return o.handleIntent(intents.Cancel{})
 		case key.Matches(msg, o.keyMap.InlineDescribe.Editor):
-			selectedRevisions := jj.NewSelectedRevisions(o.revision)
-			return o.context.RunCommand(
-				jj.SetDescription(o.revision.GetChangeId(), o.input.Value()),
-				common.Close,
-				o.context.RunInteractiveCommand(jj.Describe(selectedRevisions), common.Refresh),
-			)
+			return o.handleIntent(intents.InlineDescribeEditor{})
 		case key.Matches(msg, o.keyMap.InlineDescribe.Accept):
-			return o.context.RunCommand(jj.SetDescription(o.revision.GetChangeId(), o.input.Value()), common.Close, common.Refresh)
+			return o.handleIntent(intents.InlineDescribeAccept{})
 		}
+	case intents.Intent:
+		return o.handleIntent(msg)
 	}
 
 	o.input, cmd = o.input.Update(msg)
 
-	newValue := o.input.Value()
-	h := lipgloss.Height(newValue)
-	if h >= o.input.Height() {
-		o.SetHeight(h + 1)
-	}
-
 	return cmd
 }
 
+func (o *Operation) handleIntent(intent intents.Intent) tea.Cmd {
+	switch intent.(type) {
+	case intents.Cancel:
+		unsavedDescription := o.input.Value()
+		if o.originalDesc == "" {
+			stashed = &stashedDescription{
+				revision:    o.revision,
+				description: unsavedDescription,
+			}
+			return tea.Batch(common.Close, func() tea.Msg {
+				return intents.AddMessage{Text: "Unsaved description is stashed. Edit again to restore."}
+			})
+		}
+		return common.Close
+	case intents.InlineDescribeEditor:
+		return o.runInlineDescribeEditor()
+	case intents.InlineDescribeAccept:
+		return o.runInlineDescribeAccept()
+	default:
+		return nil
+	}
+}
+
+func (o *Operation) runInlineDescribeEditor() tea.Cmd {
+	selectedRevisions := jj.NewSelectedRevisions(o.revision)
+	cmd := jj.SetDescription(o.revision.GetChangeId(), o.input.Value())
+	return o.context.RunCommandWithInput(
+		cmd.Args, cmd.Input,
+		common.CloseApplied,
+		o.context.RunInteractiveCommand(jj.Describe(selectedRevisions), common.Refresh),
+	)
+}
+
+func (o *Operation) runInlineDescribeAccept() tea.Cmd {
+	cmd := jj.SetDescription(o.revision.GetChangeId(), o.input.Value())
+	return o.context.RunCommandWithInput(cmd.Args, cmd.Input, common.CloseApplied, common.Refresh)
+}
+
 func (o *Operation) Init() tea.Cmd {
 	return nil
 }
 
-func (o *Operation) View() string {
-	o.SetWidth(o.Parent.Width)
-	o.input.SetWidth(o.Width)
-	o.input.SetHeight(o.Height)
-	return o.input.View()
+func (o *Operation) ViewRect(dl *render.DisplayContext, box layout.Box) {
+	content := o.viewContent(box.R.Dx())
+	w, h := lipgloss.Size(content)
+	rect := cellbuf.Rect(box.R.Min.X, box.R.Min.Y, w, h)
+	dl.AddDraw(rect, content, 0)
 }
 
 func NewOperation(context *context.MainContext, revision *jj.Commit) *Operation {
 	descOutput, _ := context.RunCommandImmediate(jj.GetDescription(revision.GetChangeId()))
 	originalDesc := string(descOutput)
 	desc := originalDesc
-	if stashed != nil && stashed.revision.CommitId == revision.CommitId && stashed.description != originalDesc {
+	if stashed != nil && stashed.revision.CommitId == revision.CommitId && originalDesc == "" {
 		desc = stashed.description
 	}
 
 	// clear the stashed description regardless
 	stashed = nil
 
-	h := lipgloss.Height(desc)
-
 	selectedStyle := common.DefaultPalette.Get("revisions selected")
 
 	input := textarea.New()
 	input.CharLimit = 0
-	input.MaxHeight = 10
 	input.Prompt = ""
 	input.ShowLineNumbers = false
 	input.FocusedStyle.Base = selectedStyle.Underline(false).Strikethrough(false).Reverse(false).Blink(false)
@@ -146,7 +193,6 @@ func NewOperation(context *context.MainContext, revision *jj.Commit) *Operation
 	input.Focus()
 
 	return &Operation{
-		ViewNode:     common.NewViewNode(0, h+1),
 		context:      context,
 		keyMap:       config.Current.GetKeyMap(),
 		input:        input,
@@ -154,3 +200,18 @@ func NewOperation(context *context.MainContext, revision *jj.Commit) *Operation
 		revision:     revision,
 	}
 }
+
+func (o *Operation) viewContent(width int) string {
+	if width <= 0 {
+		width = 80
+	}
+	h := lipgloss.Height(o.input.Value())
+	if h <= 0 {
+		h = 1
+	}
+	height := h + 1
+
+	o.input.SetWidth(width)
+	o.input.SetHeight(height)
+	return o.input.View()
+}
