@@ -2,9 +2,7 @@ package evolog
 
 import (
 	"testing"
-	"time"
 
-	"github.com/charmbracelet/x/exp/teatest"
 	"github.com/idursun/jjui/internal/jj"
 	"github.com/idursun/jjui/test"
 	"github.com/stretchr/testify/assert"
@@ -41,7 +39,7 @@ func TestNewOperation_Mode(t *testing.T) {
 		t.Run(args.name, func(t *testing.T) {
 			commandRunner := test.NewTestCommandRunner(t)
 			context := test.NewTestContext(commandRunner)
-			operation := NewOperation(context, revision, 10, 20)
+			operation := NewOperation(context, revision)
 			operation.mode = args.mode
 
 			assert.Equal(t, args.isFocused, operation.IsFocused())
@@ -56,12 +54,9 @@ func TestOperation_Init(t *testing.T) {
 	defer commandRunner.Verify()
 
 	context := test.NewTestContext(commandRunner)
-	operation := NewOperation(context, revision, 10, 20)
-	tm := teatest.NewTestModel(t, operation)
+	operation := NewOperation(context, revision)
 
-	teatest.WaitFor(t, tm.Output(), func(bts []byte) bool {
-		return commandRunner.IsVerified()
-	})
-	tm.Quit()
-	tm.WaitFinished(t, teatest.WithFinalTimeout(3*time.Second))
+	test.SimulateModel(operation, operation.Init())
+
+	assert.True(t, commandRunner.IsVerified())
 }
