@@ -9,11 +9,16 @@ import (
 	"github.com/charmbracelet/bubbles/key"
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/lipgloss"
+	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/config"
 	"github.com/idursun/jjui/internal/jj"
 	"github.com/idursun/jjui/internal/ui/common"
 	"github.com/idursun/jjui/internal/ui/context"
+	"github.com/idursun/jjui/internal/ui/intents"
+	"github.com/idursun/jjui/internal/ui/layout"
 	"github.com/idursun/jjui/internal/ui/operations"
+	"github.com/idursun/jjui/internal/ui/operations/target_picker"
+	"github.com/idursun/jjui/internal/ui/render"
 )
 
 type Source int
@@ -42,7 +47,7 @@ var (
 	targetToFlags = map[Target]string{
 		TargetAfter:       "--insert-after",
 		TargetBefore:      "--insert-before",
-		TargetDestination: "--destination",
+		TargetDestination: "--onto",
 	}
 )
 
@@ -58,6 +63,8 @@ type styles struct {
 var (
 	_ operations.Operation = (*Operation)(nil)
 	_ common.Focusable     = (*Operation)(nil)
+	_ common.Overlay       = (*Operation)(nil)
+	_ common.Editable      = (*Operation)(nil)
 )
 
 type Operation struct {
@@ -67,6 +74,8 @@ type Operation struct {
 	To             *jj.Commit
 	Source         Source
 	Target         Target
+	targetName     string
+	targetPicker   *target_picker.Model
 	keyMap         config.KeyMappings[key.Binding]
 	highlightedIds []string
 	styles         styles
@@ -83,60 +92,134 @@ func (r *Operation) IsFocused() bool {
 	return true
 }
 
+func (r *Operation) IsEditing() bool {
+	return r.targetPicker != nil
+}
+
+func (r *Operation) IsOverlay() bool {
+	return r.targetPicker != nil
+}
+
 func (r *Operation) Init() tea.Cmd {
 	return nil
 }
 
 func (r *Operation) Update(msg tea.Msg) tea.Cmd {
-	if msg, ok := msg.(updateHighlightedIdsMsg); ok {
+	switch msg := msg.(type) {
+	case target_picker.TargetSelectedMsg:
+		r.targetPicker = nil
+		r.targetName = strings.TrimSpace(msg.Target)
+		return r.handleIntent(intents.Apply{Force: msg.Force})
+	case target_picker.TargetPickerCancelMsg:
+		r.targetPicker = nil
+		return nil
+	case updateHighlightedIdsMsg:
 		r.highlightedIds = msg.ids
 		return nil
-	}
-
-	if msg, ok := msg.(tea.KeyMsg); ok {
+	case intents.Intent:
+		return r.handleIntent(msg)
+	case tea.KeyMsg:
+		if r.targetPicker != nil {
+			return r.targetPicker.Update(msg)
+		}
 		return r.HandleKey(msg)
+	default:
+		if r.targetPicker != nil {
+			return r.targetPicker.Update(msg)
+		}
+		return nil
+	}
+}
+
+func (r *Operation) handleIntent(intent intents.Intent) tea.Cmd {
+	switch msg := intent.(type) {
+	case intents.StartAceJump:
+		return common.StartAceJump()
+	case intents.RebaseSetSource:
+		r.Source = rebaseSourceFromIntent(msg.Source)
+	case intents.RebaseSetTarget:
+		r.Target = rebaseTargetFromIntent(msg.Target)
+		if r.Target == TargetInsert {
+			r.InsertStart = r.To
+		}
+	case intents.RebaseToggleSkipEmptied:
+		r.SkipEmptied = !r.SkipEmptied
+	case intents.Apply:
+		skipEmptied := r.SkipEmptied
+		if r.Target == TargetInsert {
+			insertAfter := r.InsertStart.GetChangeId()
+			insertBefore := r.targetArg()
+			return r.context.RunCommand(jj.RebaseInsert(r.From, insertAfter, insertBefore, skipEmptied, msg.Force), common.RefreshAndSelect(r.From.Last()), common.Close)
+		}
+		source := sourceToFlags[r.Source]
+		target := targetToFlags[r.Target]
+		return r.context.RunCommand(jj.Rebase(r.From, r.targetArg(), source, target, skipEmptied, msg.Force), common.RefreshAndSelect(r.From.Last()), common.Close)
+	case intents.Cancel:
+		return common.Close
+	default:
+		return nil
 	}
 	return nil
 }
 
-func (r *Operation) View() string {
-	return ""
+func rebaseSourceFromIntent(source intents.RebaseSource) Source {
+	switch source {
+	case intents.RebaseSourceRevision:
+		return SourceRevision
+	case intents.RebaseSourceBranch:
+		return SourceBranch
+	case intents.RebaseSourceDescendants:
+		return SourceDescendants
+	default:
+		return SourceRevision
+	}
+}
+
+func rebaseTargetFromIntent(target intents.RebaseTarget) Target {
+	switch target {
+	case intents.RebaseTargetDestination:
+		return TargetDestination
+	case intents.RebaseTargetAfter:
+		return TargetAfter
+	case intents.RebaseTargetBefore:
+		return TargetBefore
+	case intents.RebaseTargetInsert:
+		return TargetInsert
+	default:
+		return TargetDestination
+	}
 }
 
 func (r *Operation) HandleKey(msg tea.KeyMsg) tea.Cmd {
+	if r.targetPicker != nil {
+		return r.targetPicker.Update(msg)
+	}
 	switch {
+	case key.Matches(msg, r.keyMap.AceJump):
+		return r.handleIntent(intents.StartAceJump{})
 	case key.Matches(msg, r.keyMap.Rebase.Revision):
-		r.Source = SourceRevision
+		return r.handleIntent(intents.RebaseSetSource{Source: intents.RebaseSourceRevision})
 	case key.Matches(msg, r.keyMap.Rebase.Branch):
-		r.Source = SourceBranch
+		return r.handleIntent(intents.RebaseSetSource{Source: intents.RebaseSourceBranch})
 	case key.Matches(msg, r.keyMap.Rebase.Source):
-		r.Source = SourceDescendants
+		return r.handleIntent(intents.RebaseSetSource{Source: intents.RebaseSourceDescendants})
+	case key.Matches(msg, r.keyMap.Rebase.Target):
+		r.targetPicker = target_picker.NewModel(r.context)
+		return r.targetPicker.Init()
 	case key.Matches(msg, r.keyMap.Rebase.Onto):
-		r.Target = TargetDestination
+		return r.handleIntent(intents.RebaseSetTarget{Target: intents.RebaseTargetDestination})
 	case key.Matches(msg, r.keyMap.Rebase.After):
-		r.Target = TargetAfter
+		return r.handleIntent(intents.RebaseSetTarget{Target: intents.RebaseTargetAfter})
 	case key.Matches(msg, r.keyMap.Rebase.Before):
-		r.Target = TargetBefore
+		return r.handleIntent(intents.RebaseSetTarget{Target: intents.RebaseTargetBefore})
 	case key.Matches(msg, r.keyMap.Rebase.Insert):
-		r.Target = TargetInsert
-		r.InsertStart = r.To
-	case key.Matches(msg, r.keyMap.Rebase.Insert):
-		r.Target = TargetInsert
-		r.InsertStart = r.To
+		return r.handleIntent(intents.RebaseSetTarget{Target: intents.RebaseTargetInsert})
 	case key.Matches(msg, r.keyMap.Rebase.SkipEmptied):
-		r.SkipEmptied = !r.SkipEmptied
+		return r.handleIntent(intents.RebaseToggleSkipEmptied{})
 	case key.Matches(msg, r.keyMap.Apply, r.keyMap.ForceApply):
-		ignoreImmutable := key.Matches(msg, r.keyMap.ForceApply)
-		skipEmptied := r.SkipEmptied
-		if r.Target == TargetInsert {
-			return r.context.RunCommand(jj.RebaseInsert(r.From, r.InsertStart.GetChangeId(), r.To.GetChangeId(), skipEmptied, ignoreImmutable), common.RefreshAndSelect(r.From.Last()), common.Close)
-		} else {
-			source := sourceToFlags[r.Source]
-			target := targetToFlags[r.Target]
-			return r.context.RunCommand(jj.Rebase(r.From, r.To.GetChangeId(), source, target, skipEmptied, ignoreImmutable), common.RefreshAndSelect(r.From.Last()), common.Close)
-		}
+		return r.handleIntent(intents.Apply{Force: key.Matches(msg, r.keyMap.ForceApply)})
 	case key.Matches(msg, r.keyMap.Cancel):
-		return common.Close
+		return r.handleIntent(intents.Cancel{})
 	}
 	return nil
 }
@@ -165,9 +248,7 @@ func (r *Operation) SetSelectedRevision(commit *jj.Commit) tea.Cmd {
 		if len(ids) == 1 && ids[0] == "" {
 			ids = nil
 		}
-		return common.DeferredUpdateMsg{Fn: func() tea.Cmd {
-			return r.Update(updateHighlightedIdsMsg{ids: ids})
-		}}
+		return updateHighlightedIdsMsg{ids: ids}
 	})
 }
 
@@ -175,6 +256,7 @@ func (r *Operation) ShortHelp() []key.Binding {
 	return []key.Binding{
 		r.keyMap.Apply,
 		r.keyMap.ForceApply,
+		r.keyMap.Cancel,
 		r.keyMap.Rebase.Revision,
 		r.keyMap.Rebase.Branch,
 		r.keyMap.Rebase.Source,
@@ -182,7 +264,9 @@ func (r *Operation) ShortHelp() []key.Binding {
 		r.keyMap.Rebase.After,
 		r.keyMap.Rebase.Onto,
 		r.keyMap.Rebase.Insert,
+		r.keyMap.Rebase.Target,
 		r.keyMap.Rebase.SkipEmptied,
+		r.keyMap.AceJump,
 	}
 }
 
@@ -279,10 +363,34 @@ func (r *Operation) Render(commit *jj.Commit, pos operations.RenderPosition) str
 	)
 }
 
+func (r *Operation) RenderToDisplayContext(_ *render.DisplayContext, _ *jj.Commit, _ operations.RenderPosition, _ cellbuf.Rectangle, _ cellbuf.Position) int {
+	return 0
+}
+
+func (r *Operation) DesiredHeight(_ *jj.Commit, _ operations.RenderPosition) int {
+	return 0
+}
+
 func (r *Operation) Name() string {
 	return "rebase"
 }
 
+func (r *Operation) ViewRect(dl *render.DisplayContext, box layout.Box) {
+	if r.targetPicker != nil {
+		r.targetPicker.ViewRect(dl, box)
+	}
+}
+
+func (r *Operation) targetArg() string {
+	if strings.TrimSpace(r.targetName) != "" {
+		return r.targetName
+	}
+	if r.To != nil {
+		return r.To.GetChangeId()
+	}
+	return ""
+}
+
 func NewOperation(context *context.MainContext, from jj.SelectedRevisions, source Source, target Target) *Operation {
 	styles := styles{
 		changeId:     common.DefaultPalette.Get("rebase change_id"),
