@@ -7,11 +7,16 @@ import (
 	"github.com/charmbracelet/bubbles/key"
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/lipgloss"
+	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/config"
 	"github.com/idursun/jjui/internal/jj"
 	"github.com/idursun/jjui/internal/ui/common"
 	"github.com/idursun/jjui/internal/ui/context"
+	"github.com/idursun/jjui/internal/ui/intents"
+	"github.com/idursun/jjui/internal/ui/layout"
 	"github.com/idursun/jjui/internal/ui/operations"
+	"github.com/idursun/jjui/internal/ui/operations/target_picker"
+	"github.com/idursun/jjui/internal/ui/render"
 )
 
 type Target int
@@ -27,7 +32,7 @@ var (
 	targetToFlags = map[Target]string{
 		TargetAfter:       "--insert-after",
 		TargetBefore:      "--insert-before",
-		TargetDestination: "--destination",
+		TargetDestination: "--onto",
 	}
 )
 
@@ -42,6 +47,8 @@ type styles struct {
 
 var _ operations.Operation = (*Operation)(nil)
 var _ common.Focusable = (*Operation)(nil)
+var _ common.Overlay = (*Operation)(nil)
+var _ common.Editable = (*Operation)(nil)
 
 type Operation struct {
 	context        *context.MainContext
@@ -49,6 +56,8 @@ type Operation struct {
 	InsertStart    *jj.Commit
 	To             *jj.Commit
 	Target         Target
+	targetName     string
+	targetPicker   *target_picker.Model
 	keyMap         config.KeyMappings[key.Binding]
 	highlightedIds []string
 	styles         styles
@@ -58,58 +67,121 @@ func (r *Operation) IsFocused() bool {
 	return true
 }
 
+func (r *Operation) IsEditing() bool {
+	return r.targetPicker != nil
+}
+
+func (r *Operation) IsOverlay() bool {
+	return r.targetPicker != nil
+}
+
 func (r *Operation) Init() tea.Cmd {
 	return nil
 }
 
-func (r *Operation) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
-	if msg, ok := msg.(tea.KeyMsg); ok {
-		return r, r.HandleKey(msg)
-	}
-	return r, nil
-}
-
-func (r *Operation) View() string {
-	return ""
-}
-
-func (r *Operation) HandleKey(msg tea.KeyMsg) tea.Cmd {
-	switch {
-	case key.Matches(msg, r.keyMap.Revert.Onto):
-		r.Target = TargetDestination
-	case key.Matches(msg, r.keyMap.Revert.After):
-		r.Target = TargetAfter
-	case key.Matches(msg, r.keyMap.Revert.Before):
-		r.Target = TargetBefore
-	case key.Matches(msg, r.keyMap.Revert.Insert):
-		r.Target = TargetInsert
-		r.InsertStart = r.To
-	case key.Matches(msg, r.keyMap.Apply):
-		if r.Target == TargetInsert {
-			return r.context.RunCommand(jj.RevertInsert(r.From, r.InsertStart.GetChangeId(), r.To.GetChangeId()), common.RefreshAndSelect(r.From.Last()), common.Close)
-		} else {
-			source := "--revisions"
-			target := targetToFlags[r.Target]
-			return r.context.RunCommand(jj.Revert(r.From, r.To.GetChangeId(), source, target), common.RefreshAndSelect(r.From.Last()), common.Close)
+func (r *Operation) Update(msg tea.Msg) tea.Cmd {
+	switch msg := msg.(type) {
+	case target_picker.TargetSelectedMsg:
+		r.targetPicker = nil
+		r.targetName = strings.TrimSpace(msg.Target)
+		return r.handleIntent(intents.Apply{Force: msg.Force})
+	case target_picker.TargetPickerCancelMsg:
+		r.targetPicker = nil
+		return nil
+	case intents.Intent:
+		return r.handleIntent(msg)
+	case tea.KeyMsg:
+		if r.targetPicker != nil {
+			return r.targetPicker.Update(msg)
 		}
-	case key.Matches(msg, r.keyMap.Cancel):
+		return r.HandleKey(msg)
+	default:
+		if r.targetPicker != nil {
+			return r.targetPicker.Update(msg)
+		}
+		return nil
+	}
+}
+
+func (r *Operation) handleIntent(intent intents.Intent) tea.Cmd {
+	switch msg := intent.(type) {
+	case intents.RevertSetTarget:
+		r.Target = revertTargetFromIntent(msg.Target)
+		if r.Target == TargetInsert {
+			r.InsertStart = r.To
+		}
+	case intents.Apply:
+		if r.Target == TargetInsert {
+			insertAfter := r.InsertStart.GetChangeId()
+			insertBefore := r.targetArg()
+			return r.context.RunCommand(jj.RevertInsert(r.From, insertAfter, insertBefore), common.RefreshAndSelect(r.From.Last()), common.Close)
+		}
+		source := "--revisions"
+		target := targetToFlags[r.Target]
+		return r.context.RunCommand(jj.Revert(r.From, r.targetArg(), source, target), common.RefreshAndSelect(r.From.Last()), common.Close)
+	case intents.Cancel:
 		return common.Close
+	default:
+		return nil
 	}
 	return nil
 }
 
-func (r *Operation) SetSelectedRevision(commit *jj.Commit) {
+func revertTargetFromIntent(target intents.RevertTarget) Target {
+	switch target {
+	case intents.RevertTargetDestination:
+		return TargetDestination
+	case intents.RevertTargetAfter:
+		return TargetAfter
+	case intents.RevertTargetBefore:
+		return TargetBefore
+	case intents.RevertTargetInsert:
+		return TargetInsert
+	default:
+		return TargetDestination
+	}
+}
+
+func (r *Operation) HandleKey(msg tea.KeyMsg) tea.Cmd {
+	if r.targetPicker != nil {
+		return r.targetPicker.Update(msg)
+	}
+	switch {
+	case key.Matches(msg, r.keyMap.Revert.Onto):
+		return r.handleIntent(intents.RevertSetTarget{Target: intents.RevertTargetDestination})
+	case key.Matches(msg, r.keyMap.Revert.After):
+		return r.handleIntent(intents.RevertSetTarget{Target: intents.RevertTargetAfter})
+	case key.Matches(msg, r.keyMap.Revert.Before):
+		return r.handleIntent(intents.RevertSetTarget{Target: intents.RevertTargetBefore})
+	case key.Matches(msg, r.keyMap.Revert.Insert):
+		return r.handleIntent(intents.RevertSetTarget{Target: intents.RevertTargetInsert})
+	case key.Matches(msg, r.keyMap.Revert.Target):
+		r.targetPicker = target_picker.NewModel(r.context)
+		return r.targetPicker.Init()
+	case key.Matches(msg, r.keyMap.Apply):
+		return r.handleIntent(intents.Apply{})
+	case key.Matches(msg, r.keyMap.Cancel):
+		return r.handleIntent(intents.Cancel{})
+	}
+	return nil
+}
+
+func (r *Operation) SetSelectedRevision(commit *jj.Commit) tea.Cmd {
 	r.highlightedIds = nil
 	r.To = commit
 	r.highlightedIds = r.From.GetIds()
+	return nil
 }
 
 func (r *Operation) ShortHelp() []key.Binding {
 	return []key.Binding{
+		r.keyMap.Apply,
+		r.keyMap.Cancel,
 		r.keyMap.Revert.Before,
 		r.keyMap.Revert.After,
 		r.keyMap.Revert.Onto,
 		r.keyMap.Revert.Insert,
+		r.keyMap.Revert.Target,
 	}
 }
 
@@ -194,10 +266,34 @@ func (r *Operation) Render(commit *jj.Commit, pos operations.RenderPosition) str
 	)
 }
 
+func (r *Operation) RenderToDisplayContext(_ *render.DisplayContext, _ *jj.Commit, _ operations.RenderPosition, _ cellbuf.Rectangle, _ cellbuf.Position) int {
+	return 0
+}
+
+func (r *Operation) DesiredHeight(_ *jj.Commit, _ operations.RenderPosition) int {
+	return 0
+}
+
 func (r *Operation) Name() string {
 	return "revert"
 }
 
+func (r *Operation) ViewRect(dl *render.DisplayContext, box layout.Box) {
+	if r.targetPicker != nil {
+		r.targetPicker.ViewRect(dl, box)
+	}
+}
+
+func (r *Operation) targetArg() string {
+	if strings.TrimSpace(r.targetName) != "" {
+		return r.targetName
+	}
+	if r.To != nil {
+		return r.To.GetChangeId()
+	}
+	return ""
+}
+
 func NewOperation(context *context.MainContext, from jj.SelectedRevisions, target Target) *Operation {
 	styles := styles{
 		changeId:     common.DefaultPalette.Get("revert change_id"),
