@@ -8,11 +8,15 @@ import (
 	"github.com/charmbracelet/bubbles/key"
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/lipgloss"
+	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/config"
 	"github.com/idursun/jjui/internal/jj"
 	"github.com/idursun/jjui/internal/ui/common"
 	"github.com/idursun/jjui/internal/ui/context"
+	"github.com/idursun/jjui/internal/ui/intents"
+	"github.com/idursun/jjui/internal/ui/layout"
 	"github.com/idursun/jjui/internal/ui/operations"
+	"github.com/idursun/jjui/internal/ui/render"
 )
 
 var _ operations.Operation = (*Model)(nil)
@@ -38,20 +42,22 @@ func (m *Model) Init() tea.Cmd {
 }
 
 func (m *Model) Update(msg tea.Msg) tea.Cmd {
-	if msg, ok := msg.(tea.KeyMsg); ok {
+	switch msg := msg.(type) {
+	case intents.Intent:
+		return m.handleIntent(msg)
+	case tea.KeyMsg:
 		return m.HandleKey(msg)
 	}
 	return nil
 }
 
-func (m *Model) View() string {
-	return ""
-}
+func (m *Model) ViewRect(_ *render.DisplayContext, _ layout.Box) {}
 
 func (m *Model) ShortHelp() []key.Binding {
 	return []key.Binding{
 		m.keyMap.ToggleSelect,
 		m.keyMap.Apply,
+		m.keyMap.AceJump,
 		m.keyMap.Cancel,
 	}
 }
@@ -75,7 +81,23 @@ func (m *Model) SetSelectedRevision(commit *jj.Commit) tea.Cmd {
 
 func (m *Model) HandleKey(msg tea.KeyMsg) tea.Cmd {
 	switch {
+	case key.Matches(msg, m.keyMap.AceJump):
+		return m.handleIntent(intents.StartAceJump{})
 	case key.Matches(msg, m.keyMap.ToggleSelect):
+		return m.handleIntent(intents.SetParentsToggleSelect{})
+	case key.Matches(msg, m.keyMap.Apply):
+		return m.handleIntent(intents.Apply{})
+	case key.Matches(msg, m.keyMap.Cancel):
+		return m.handleIntent(intents.Cancel{})
+	}
+	return nil
+}
+
+func (m *Model) handleIntent(intent intents.Intent) tea.Cmd {
+	switch intent.(type) {
+	case intents.StartAceJump:
+		return common.StartAceJump()
+	case intents.SetParentsToggleSelect:
 		if m.current.GetChangeId() == m.target.GetChangeId() {
 			return nil
 		}
@@ -94,7 +116,8 @@ func (m *Model) HandleKey(msg tea.KeyMsg) tea.Cmd {
 				m.toAdd = append(m.toAdd, changeId)
 			}
 		}
-	case key.Matches(msg, m.keyMap.Apply):
+		return nil
+	case intents.Apply:
 		if len(m.toAdd) == 0 && len(m.toRemove) == 0 {
 			return common.Close
 		}
@@ -107,7 +130,7 @@ func (m *Model) HandleKey(msg tea.KeyMsg) tea.Cmd {
 		}
 
 		return m.context.RunCommand(jj.SetParents(m.target.GetChangeId(), parentsToAdd, parentsToRemove), common.RefreshAndSelect(m.target.GetChangeId()), common.Close)
-	case key.Matches(msg, m.keyMap.Cancel):
+	case intents.Cancel:
 		return common.Close
 	}
 	return nil
@@ -133,6 +156,14 @@ func (m *Model) Render(commit *jj.Commit, renderPosition operations.RenderPositi
 	return ""
 }
 
+func (m *Model) RenderToDisplayContext(_ *render.DisplayContext, _ *jj.Commit, _ operations.RenderPosition, _ cellbuf.Rectangle, _ cellbuf.Position) int {
+	return 0
+}
+
+func (m *Model) DesiredHeight(_ *jj.Commit, _ operations.RenderPosition) int {
+	return 0
+}
+
 func (m *Model) Name() string {
 	return "set parents"
 }
