@@ -5,7 +5,6 @@ import (
 	"testing"
 
 	tea "github.com/charmbracelet/bubbletea"
-	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/jj"
 	"github.com/idursun/jjui/internal/ui/common"
 	"github.com/idursun/jjui/test"
@@ -19,10 +18,8 @@ func TestConfirm(t *testing.T) {
 	defer commandRunner.Verify()
 
 	model := NewModel(test.NewTestContext(commandRunner))
-	model.SetFrame(cellbuf.Rect(0, 0, 100, 20))
-	model.Parent = common.NewViewNode(100, 20)
 	test.SimulateModel(model, model.Init())
-	assert.Contains(t, model.View(), "redo")
+	assert.Contains(t, test.RenderImmediate(model, 100, 20), "redo")
 
 	test.SimulateModel(model, test.Press(tea.KeyEnter))
 }
@@ -33,10 +30,8 @@ func TestCancel(t *testing.T) {
 	defer commandRunner.Verify()
 
 	model := NewModel(test.NewTestContext(commandRunner))
-	model.SetFrame(cellbuf.Rect(0, 0, 100, 20))
-	model.Parent = common.NewViewNode(100, 20)
 	test.SimulateModel(model, model.Init())
-	assert.Contains(t, model.View(), "redo")
+	assert.Contains(t, test.RenderImmediate(model, 100, 20), "redo")
 
 	test.SimulateModel(model, test.Press(tea.KeyEsc))
 }
@@ -48,8 +43,6 @@ func TestRedoNothingToRedo(t *testing.T) {
 	defer commandRunner.Verify()
 
 	model := NewModel(test.NewTestContext(commandRunner))
-	model.SetFrame(cellbuf.Rect(0, 0, 100, 20))
-	model.Parent = common.NewViewNode(100, 20)
 	test.SimulateModel(model, model.Init())
 
 	var msgs []tea.Msg
