@@ -12,12 +12,13 @@ import (
 	"strings"
 	"sync/atomic"
 
-	"github.com/idursun/jjui/internal/ui/common/list"
 	"github.com/idursun/jjui/internal/ui/intents"
+	"github.com/idursun/jjui/internal/ui/layout"
 	"github.com/idursun/jjui/internal/ui/operations/ace_jump"
 	"github.com/idursun/jjui/internal/ui/operations/duplicate"
 	"github.com/idursun/jjui/internal/ui/operations/revert"
 	"github.com/idursun/jjui/internal/ui/operations/set_parents"
+	"github.com/idursun/jjui/internal/ui/render"
 
 	"github.com/idursun/jjui/internal/parser"
 	"github.com/idursun/jjui/internal/ui/operations/describe"
@@ -41,56 +42,61 @@ import (
 )
 
 var (
-	_ list.IList         = (*Model)(nil)
-	_ list.IListCursor   = (*Model)(nil)
-	_ common.Focusable   = (*Model)(nil)
-	_ common.Editable    = (*Model)(nil)
-	_ common.IMouseAware = (*Model)(nil)
-)
-
-var (
-	pageDownKey = key.NewBinding(key.WithKeys("pgdown"))
-	pageUpKey   = key.NewBinding(key.WithKeys("pgup"))
+	_ common.Focusable      = (*Model)(nil)
+	_ common.Editable       = (*Model)(nil)
+	_ common.ImmediateModel = (*Model)(nil)
 )
 
 type Model struct {
-	*common.ViewNode
-	*common.MouseAware
-	rows             []parser.Row
-	tag              atomic.Uint64
-	revisionToSelect string
-	offScreenRows    []parser.Row
-	streamer         *graph.GraphStreamer
-	hasMore          bool
-	op               common.Model
-	cursor           int
-	context          *appContext.MainContext
-	keymap           config.KeyMappings[key.Binding]
-	output           string
-	err              error
-	quickSearch      string
-	previousOpLogId  string
-	isLoading        bool
-	renderer         *revisionListRenderer
-	textStyle        lipgloss.Style
-	dimmedStyle      lipgloss.Style
-	selectedStyle    lipgloss.Style
-	matchedStyle     lipgloss.Style
-	ensureCursorView bool
-	requestInFlight  bool
+	rows                   []parser.Row
+	tag                    atomic.Uint64
+	revisionToSelect       string
+	offScreenRows          []parser.Row
+	streamer               *graph.GraphStreamer
+	hasMore                bool
+	op                     common.ImmediateModel
+	cursor                 int
+	context                *appContext.MainContext
+	keymap                 config.KeyMappings[key.Binding]
+	output                 string
+	err                    error
+	quickSearch            string
+	previousOpLogId        string
+	isLoading              bool
+	displayContextRenderer *DisplayContextRenderer
+	textStyle              lipgloss.Style
+	dimmedStyle            lipgloss.Style
+	selectedStyle          lipgloss.Style
+	matchedStyle           lipgloss.Style
+	ensureCursorView       bool
+	requestInFlight        bool
 }
 
 type revisionsMsg struct {
 	msg tea.Msg
 }
 
-// Allow a message to be targetted to this component.
 func RevisionsCmd(msg tea.Msg) tea.Cmd {
 	return func() tea.Msg {
 		return revisionsMsg{msg: msg}
 	}
 }
 
+type ItemClickedMsg struct {
+	Index int
+}
+
+type ViewportScrollMsg struct {
+	Delta      int
+	Horizontal bool
+}
+
+func (v ViewportScrollMsg) SetDelta(delta int, horizontal bool) tea.Msg {
+	v.Delta = delta
+	v.Horizontal = horizontal
+	return v
+}
+
 type updateRevisionsMsg struct {
 	rows             []parser.Row
 	selectedRevision string
@@ -118,59 +124,22 @@ func (m *Model) SetCursor(index int) {
 	}
 }
 
-func (m *Model) ClickAt(x, y int) tea.Cmd {
-	if len(m.rows) == 0 {
-		return nil
-	}
-
-	localY := y - m.Frame.Min.Y
-
-	currentStart := m.renderer.ViewRange.Start
-	if localY >= m.Height {
-		localY = m.Height - 1
-		if localY < 0 {
-			return nil
-		}
-	}
-	line := currentStart + localY
-	row := m.rowAtLine(line)
-	if row == -1 {
-		return nil
-	}
-
-	m.SetCursor(row)
-	return m.updateSelection()
-}
-
-func (m *Model) rowAtLine(line int) int {
-	for _, rr := range m.renderer.RowRanges() {
-		if line >= rr.StartLine && line < rr.EndLine {
-			return rr.Row
-		}
-	}
-	return -1
+func (m *Model) HasMore() bool {
+	return m.hasMore
 }
 
 func (m *Model) Scroll(delta int) tea.Cmd {
 	m.ensureCursorView = false
-	desiredStart := m.renderer.ViewRange.Start + delta
-	if desiredStart < 0 {
-		desiredStart = 0
-	}
+	currentStart := m.displayContextRenderer.GetScrollOffset()
+	desiredStart := currentStart + delta
+	m.displayContextRenderer.SetScrollOffset(desiredStart)
 
-	totalLines := m.renderer.AbsoluteLineCount()
-	maxStart := totalLines - m.Height
-	if maxStart < 0 {
-		maxStart = 0
-	}
-	newStart := desiredStart
-	if newStart > maxStart {
-		newStart = maxStart
-	}
-	m.renderer.ViewRange.Start = newStart
-
-	if m.hasMore && (desiredStart > maxStart || newStart+m.Height >= totalLines-1) {
-		return m.requestMoreRows(m.tag.Load())
+	// Request more rows if scrolling down and near the end
+	if m.hasMore && delta > 0 {
+		lastRowIndex := m.displayContextRenderer.GetLastRowIndex()
+		if lastRowIndex >= len(m.rows)-1 {
+			return m.requestMoreRows(m.tag.Load())
+		}
 	}
 	return nil
 }
@@ -179,31 +148,6 @@ func (m *Model) Len() int {
 	return len(m.rows)
 }
 
-func (m *Model) GetItemRenderer(index int) list.IItemRenderer {
-	row := m.rows[index]
-	inLane := m.renderer.tracer.IsInSameLane(index)
-	isHighlighted := index == m.cursor
-
-	return &itemRenderer{
-		row:           row,
-		isHighlighted: isHighlighted,
-		SearchText:    m.quickSearch,
-		textStyle:     m.textStyle,
-		dimmedStyle:   m.dimmedStyle,
-		selectedStyle: m.selectedStyle,
-		matchedStyle:  m.matchedStyle,
-		isChecked:     m.renderer.selections[row.Commit.GetChangeId()],
-		isGutterInLane: func(lineIndex, segmentIndex int) bool {
-			return m.renderer.tracer.IsGutterInLane(index, lineIndex, segmentIndex)
-		},
-		updateGutterText: func(lineIndex, segmentIndex int, text string) string {
-			return m.renderer.tracer.UpdateGutterText(index, lineIndex, segmentIndex, text)
-		},
-		inLane: inLane,
-		op:     m.op.(operations.Operation),
-	}
-}
-
 func (m *Model) IsEditing() bool {
 	if f, ok := m.op.(common.Editable); ok {
 		return f.IsEditing()
@@ -289,32 +233,38 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case intents.Intent:
 		return m.handleIntent(msg)
-	case tea.MouseMsg:
-		switch msg.Action {
-		case tea.MouseActionPress:
-			switch msg.Button {
-			case tea.MouseButtonLeft:
-				if !m.InNormalMode() {
-					return nil
-				}
-				return m.ClickAt(msg.X, msg.Y)
-			case tea.MouseButtonWheelUp:
-				return m.Scroll(-3)
-			case tea.MouseButtonWheelDown:
-				return m.Scroll(3)
-			}
+	case ItemClickedMsg:
+		// Don't allow changing selection if the operation is editing (e.g. describe)
+		if editable, ok := m.op.(common.Editable); ok && editable.IsEditing() {
 			return nil
 		}
+		// Don't allow changing selection if the operation is an overlay (e.g. details)
+		if overlay, ok := m.op.(common.Overlay); ok && overlay.IsOverlay() {
+			return nil
+		}
+		m.SetCursor(msg.Index)
+		return m.updateSelection()
+	case ViewportScrollMsg:
+		if msg.Horizontal {
+			return nil
+		}
+		return m.Scroll(msg.Delta)
 
 	case common.CloseViewMsg:
 		m.op = operations.NewDefault()
 		return m.updateSelection()
+	case common.RestoreOperationMsg:
+		if op, ok := msg.Operation.(operations.Operation); ok {
+			m.op = op
+			return m.updateSelection()
+		}
+		m.op = operations.NewDefault()
+		return m.updateSelection()
 	case common.QuickSearchMsg:
 		m.quickSearch = strings.ToLower(string(msg))
-		m.SetCursor(m.search(0))
+		m.SetCursor(m.search(0, false))
 		m.op = operations.NewDefault()
-		m.renderer.Reset()
-		return nil
+		return m.updateSelection()
 	case common.CommandCompletedMsg:
 		m.output = msg.Output
 		m.err = msg.Err
@@ -368,7 +318,8 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 
 		if m.hasMore {
 			// keep requesting rows until we reach the initial load count or the current cursor position
-			if len(m.offScreenRows) < m.cursor+1 || len(m.offScreenRows) < m.renderer.ViewRange.LastRowIndex+1 {
+			lastRowIndex := m.displayContextRenderer.GetLastRowIndex()
+			if len(m.offScreenRows) < m.cursor+1 || len(m.offScreenRows) < lastRowIndex+1 {
 				return m.requestMoreRows(msg.tag)
 			}
 		} else if m.streamer != nil {
@@ -399,6 +350,18 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 		return tea.Batch(cmds...)
 	}
 
+	if intent, ok := msg.(intents.Intent); ok {
+		if cmd := m.handleIntent(intent); cmd != nil {
+			return cmd
+		}
+		return m.op.Update(msg)
+	}
+
+	// Non-input messages are broadcast to the current operation
+	if !common.IsInputMessage(msg) {
+		return m.op.Update(msg)
+	}
+
 	if len(m.rows) == 0 {
 		return nil
 	}
@@ -415,10 +378,10 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case tea.KeyMsg:
 		switch {
-		case key.Matches(msg, m.keymap.Up, pageUpKey):
-			return m.handleIntent(intents.Navigate{Delta: -1, Page: key.Matches(msg, pageUpKey)})
-		case key.Matches(msg, m.keymap.Down, pageDownKey):
-			return m.handleIntent(intents.Navigate{Delta: 1, Page: key.Matches(msg, pageDownKey)})
+		case key.Matches(msg, m.keymap.Up, m.keymap.ScrollUp):
+			return m.handleIntent(intents.Navigate{Delta: -1, IsPage: key.Matches(msg, m.keymap.ScrollUp)})
+		case key.Matches(msg, m.keymap.Down, m.keymap.ScrollDown):
+			return m.handleIntent(intents.Navigate{Delta: 1, IsPage: key.Matches(msg, m.keymap.ScrollDown)})
 		case key.Matches(msg, m.keymap.JumpToParent):
 			return m.handleIntent(intents.Navigate{Target: intents.TargetParent})
 		case key.Matches(msg, m.keymap.JumpToChildren):
@@ -426,11 +389,7 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 		case key.Matches(msg, m.keymap.JumpToWorkingCopy):
 			return m.handleIntent(intents.Navigate{Target: intents.TargetWorkingCopy})
 		case key.Matches(msg, m.keymap.AceJump):
-			op := ace_jump.NewOperation(m, func(index int) parser.Row {
-				return m.rows[index]
-			}, m.renderer.FirstRowIndex, m.renderer.LastRowIndex)
-			m.op = op
-			return op.Init()
+			return m.handleIntent(intents.StartAceJump{})
 		default:
 			if op, ok := m.op.(common.Focusable); ok && op.IsFocused() {
 				return m.op.Update(msg)
@@ -438,24 +397,16 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 
 			switch {
 			case m.quickSearch != "" && (msg.Type == tea.KeyEsc || msg.Type == tea.KeyEnter):
-				m.quickSearch = ""
-				m.renderer.Reset()
-				return nil
+				return m.handleIntent(intents.RevisionsQuickSearchClear{})
 			case key.Matches(msg, m.keymap.ToggleSelect):
-				commit := m.rows[m.cursor].Commit
-				changeId := commit.GetChangeId()
-				item := appContext.SelectedRevision{ChangeId: changeId, CommitId: commit.CommitId}
-				m.context.ToggleCheckedItem(item)
-				m.jumpToParent(jj.NewSelectedRevisions(commit))
+				return m.handleIntent(intents.RevisionsToggleSelect{})
 			case key.Matches(msg, m.keymap.Cancel):
-				m.context.ClearCheckedItems(reflect.TypeFor[appContext.SelectedRevision]())
-				m.op = operations.NewDefault()
-				m.renderer.Reset()
-				return nil
-			case key.Matches(msg, m.keymap.QuickSearchCycle):
-				m.SetCursor(m.search(m.cursor + 1))
-				m.renderer.Reset()
-				return nil
+				return m.handleIntent(intents.Cancel{})
+			case m.quickSearch != "" &&
+				(key.Matches(msg, m.keymap.QuickSearchNext) ||
+					key.Matches(msg, m.keymap.QuickSearchPrev)):
+				reverse := key.Matches(msg, m.keymap.QuickSearchPrev)
+				return m.handleIntent(intents.QuickSearchCycle{Reverse: reverse})
 			case key.Matches(msg, m.keymap.Details.Mode):
 				return m.handleIntent(intents.OpenDetails{})
 			case key.Matches(msg, m.keymap.InlineDescribe.Mode):
@@ -474,8 +425,7 @@ func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 			case key.Matches(msg, m.keymap.Abandon):
 				return m.handleIntent(intents.StartAbandon{})
 			case key.Matches(msg, m.keymap.Bookmark.Set):
-				m.op = bookmark.NewSetBookmarkOperation(m.context, m.SelectedRevision().GetChangeId())
-				return m.op.Init()
+				return m.handleIntent(intents.BookmarksSet{})
 			case key.Matches(msg, m.keymap.Split, m.keymap.SplitParallel):
 				return m.handleIntent(intents.StartSplit{
 					IsParallel: key.Matches(msg, m.keymap.SplitParallel),
@@ -531,6 +481,15 @@ func (m *Model) handleIntent(intent intents.Intent) tea.Cmd {
 		return m.startDuplicate(intent)
 	case intents.SetParents:
 		return m.startSetParents(intent)
+	case intents.BookmarksSet:
+		return m.startBookmarkSet()
+	case intents.RevisionsToggleSelect:
+		commit := m.rows[m.cursor].Commit
+		changeId := commit.GetChangeId()
+		item := appContext.SelectedRevision{ChangeId: changeId, CommitId: commit.CommitId}
+		m.context.ToggleCheckedItem(item)
+		m.jumpToParent(jj.NewSelectedRevisions(commit))
+		return nil
 	case intents.Navigate:
 		return m.navigate(intent)
 	case intents.StartDescribe:
@@ -545,10 +504,41 @@ func (m *Model) handleIntent(intent intents.Intent) tea.Cmd {
 		return m.startRebase(intent)
 	case intents.Refresh:
 		return m.refresh(intent)
+	case intents.Cancel:
+		m.context.ClearCheckedItems(reflect.TypeFor[appContext.SelectedRevision]())
+		m.op = operations.NewDefault()
+		return nil
+	case intents.QuickSearchCycle:
+		offset := 1
+		if intent.Reverse {
+			offset = -1
+		}
+		m.SetCursor(m.search(m.cursor+offset, intent.Reverse))
+		return m.updateSelection()
+	case intents.RevisionsQuickSearchClear:
+		m.quickSearch = ""
+		return nil
+	case intents.StartAceJump:
+		parentOp := m.op
+		// Create ace jump with parent operation
+		op := ace_jump.NewOperation(m.SetCursor, func(index int) parser.Row {
+			return m.rows[index]
+		}, m.displayContextRenderer.GetFirstRowIndex(), m.displayContextRenderer.GetLastRowIndex(), parentOp)
+		m.op = op
+		return op.Init()
 	}
 	return nil
 }
 
+func (m *Model) startBookmarkSet() tea.Cmd {
+	rev := m.SelectedRevision()
+	if rev == nil {
+		return nil
+	}
+	m.op = bookmark.NewSetBookmarkOperation(m.context, rev.GetChangeId())
+	return m.op.Init()
+}
+
 func (m *Model) refresh(intent intents.Refresh) tea.Cmd {
 	if !intent.KeepSelections {
 		m.context.ClearCheckedItems(reflect.TypeFor[appContext.SelectedRevision]())
@@ -566,7 +556,6 @@ func (m *Model) openDetails(_ intents.OpenDetails) tea.Cmd {
 		return nil
 	}
 	model := details.NewOperation(m.context, m.SelectedRevision())
-	model.Parent = m.ViewNode
 	m.op = model
 	return m.op.Init()
 }
@@ -588,7 +577,7 @@ func (m *Model) startSquash(intent intents.StartSquash) tea.Cmd {
 		m.SetCursor(m.cursor + 1)
 	}
 	m.op = squash.NewOperation(m.context, selected, squash.WithFiles(intent.Files))
-	return m.op.Init()
+	return tea.Batch(m.op.Init(), m.updateSelection())
 }
 
 func (m *Model) startRebase(intent intents.StartRebase) tea.Cmd {
@@ -600,14 +589,8 @@ func (m *Model) startRebase(intent intents.StartRebase) tea.Cmd {
 		return nil
 	}
 
-	source := intent.Source
-	if source == 0 {
-		source = rebase.SourceRevision
-	}
-	target := intent.Target
-	if target == 0 {
-		target = rebase.TargetDestination
-	}
+	source := rebaseSourceFromIntent(intent.Source)
+	target := rebaseTargetFromIntent(intent.Target)
 	m.op = rebase.NewOperation(m.context, selected, source, target)
 	return m.op.Init()
 }
@@ -621,14 +604,48 @@ func (m *Model) startRevert(intent intents.StartRevert) tea.Cmd {
 		return nil
 	}
 
-	target := intent.Target
-	if target == 0 {
-		target = revert.TargetDestination
-	}
+	target := revertTargetFromIntent(intent.Target)
 	m.op = revert.NewOperation(m.context, selected, target)
 	return m.op.Init()
 }
 
+func rebaseSourceFromIntent(source intents.RebaseSource) rebase.Source {
+	switch source {
+	case intents.RebaseSourceBranch:
+		return rebase.SourceBranch
+	case intents.RebaseSourceDescendants:
+		return rebase.SourceDescendants
+	default:
+		return rebase.SourceRevision
+	}
+}
+
+func rebaseTargetFromIntent(target intents.RebaseTarget) rebase.Target {
+	switch target {
+	case intents.RebaseTargetAfter:
+		return rebase.TargetAfter
+	case intents.RebaseTargetBefore:
+		return rebase.TargetBefore
+	case intents.RebaseTargetInsert:
+		return rebase.TargetInsert
+	default:
+		return rebase.TargetDestination
+	}
+}
+
+func revertTargetFromIntent(target intents.RevertTarget) revert.Target {
+	switch target {
+	case intents.RevertTargetAfter:
+		return revert.TargetAfter
+	case intents.RevertTargetBefore:
+		return revert.TargetBefore
+	case intents.RevertTargetInsert:
+		return revert.TargetInsert
+	default:
+		return revert.TargetDestination
+	}
+}
+
 func (m *Model) startDuplicate(intent intents.StartDuplicate) tea.Cmd {
 	selected := intent.Selected
 	if len(selected.Revisions) == 0 {
@@ -759,15 +776,17 @@ func (m *Model) navigate(intent intents.Navigate) tea.Cmd {
 		return m.updateSelection()
 	}
 
-	step := intent.Delta
-	if step == 0 {
-		step = 1
+	delta := intent.Delta
+	if delta == 0 {
+		delta = 1
 	}
-	if intent.Page {
-		span := m.renderer.LastRowIndex - m.renderer.FirstRowIndex - 1
-		if span < 1 {
-			span = 1
-		}
+
+	// Calculate step (convert page scroll to item count)
+	step := delta
+	if intent.IsPage {
+		firstRowIndex := m.displayContextRenderer.GetFirstRowIndex()
+		lastRowIndex := m.displayContextRenderer.GetLastRowIndex()
+		span := max(lastRowIndex-firstRowIndex-1, 1)
 		if step < 0 {
 			step = -span
 		} else {
@@ -775,35 +794,26 @@ func (m *Model) navigate(intent intents.Navigate) tea.Cmd {
 		}
 	}
 
+	// Calculate new cursor position
+	totalItems := len(m.rows)
+	newCursor := m.cursor + step
+
 	if step > 0 {
-		if len(m.rows) > 0 && m.cursor == len(m.rows)-1 && !m.hasMore && step > 1 {
-			return func() tea.Msg {
-				return common.CommandCompletedMsg{
-					Output: fmt.Sprintf("Already at the bottom of revset `%s`", m.context.CurrentRevset),
-					Err:    nil,
-				}
+		// Moving down
+		if newCursor >= totalItems {
+			if allowStream && m.hasMore {
+				return m.requestMoreRows(m.tag.Load())
 			}
+			newCursor = totalItems - 1
 		}
-		if m.cursor+step < len(m.rows) {
-			m.SetCursor(m.cursor + step)
-		} else if allowStream && m.hasMore {
-			return m.requestMoreRows(m.tag.Load())
-		} else if len(m.rows) > 0 {
-			m.SetCursor(len(m.rows) - 1)
+	} else {
+		// Moving up
+		if newCursor < 0 {
+			newCursor = 0
 		}
-	} else if step < 0 {
-		amount := -step
-		if m.cursor == 0 && amount > 1 {
-			return func() tea.Msg {
-				return common.CommandCompletedMsg{
-					Output: fmt.Sprintf("Already at the top of revset `%s`", m.context.CurrentRevset),
-					Err:    nil,
-				}
-			}
-		}
-		m.SetCursor(max(m.cursor-amount, 0))
 	}
 
+	m.SetCursor(newCursor)
 	m.ensureCursorView = ensureView
 	return m.updateSelection()
 }
@@ -828,7 +838,6 @@ func (m *Model) startEvolog(intent intents.StartEvolog) tea.Cmd {
 		return nil
 	}
 	model := evolog.NewOperation(m.context, commit)
-	model.Parent = m.ViewNode
 	m.op = model
 	return m.op.Init()
 }
@@ -842,7 +851,6 @@ func (m *Model) startInlineDescribe(intent intents.StartInlineDescribe) tea.Cmd
 		return nil
 	}
 	model := describe.NewOperation(m.context, commit)
-	model.Parent = m.ViewNode
 	m.op = model
 	return m.op.Init()
 }
@@ -870,7 +878,7 @@ func (m *Model) startSplit(intent intents.StartSplit) tea.Cmd {
 	if commit == nil {
 		return nil
 	}
-	return m.context.RunInteractiveCommand(jj.Split(commit.GetChangeId(), intent.Files, intent.IsParallel), common.Refresh)
+	return m.context.RunInteractiveCommand(jj.Split(commit.GetChangeId(), intent.Files, intent.IsParallel, intent.IsInteractive), common.Refresh)
 }
 
 func (m *Model) updateSelection() tea.Cmd {
@@ -939,26 +947,51 @@ func (m *Model) updateGraphRows(rows []parser.Row, selectedRevision string) {
 	}
 }
 
-func (m *Model) View() string {
+func (m *Model) ViewRect(dl *render.DisplayContext, box layout.Box) {
 	if len(m.rows) == 0 {
+		content := ""
 		if m.isLoading {
-			return lipgloss.Place(m.Width, m.Height, lipgloss.Center, lipgloss.Center, "loading")
+			content = lipgloss.Place(box.R.Dx(), box.R.Dy(), lipgloss.Center, lipgloss.Center, "loading")
+		} else {
+			content = lipgloss.Place(box.R.Dx(), box.R.Dy(), lipgloss.Center, lipgloss.Center, "(no matching revisions)")
 		}
-		return lipgloss.Place(m.Width, m.Height, lipgloss.Center, lipgloss.Center, "(no matching revisions)")
+		dl.AddDraw(box.R, content, 0)
+		return
 	}
 
-	if config.Current.UI.Tracer.Enabled {
-		start, end := m.renderer.FirstRowIndex, m.renderer.LastRowIndex+1 // +1 because the last row is inclusive in the view range
-		log.Println("Visible row range:", start, end, "Cursor:", m.cursor, "Total rows:", len(m.rows))
-		m.renderer.tracer = parser.NewTracer(m.rows, m.cursor, start, end)
-	} else {
-		m.renderer.tracer = parser.NewNoopTracer()
+	// Set selections
+	m.displayContextRenderer.SetSelections(m.context.GetSelectedRevisions())
+
+	// Get operation if any
+	var op operations.Operation
+	if opModel, ok := m.op.(operations.Operation); ok {
+		op = opModel
 	}
 
-	m.renderer.selections = m.context.GetSelectedRevisions()
+	// Render to DisplayContext
+	m.displayContextRenderer.Render(
+		dl,
+		m.rows,
+		m.cursor,
+		box,
+		op,
+		m.quickSearch,
+		m.ensureCursorView,
+	)
 
-	output := m.renderer.RenderWithOptions(list.RenderOptions{FocusIndex: m.cursor, EnsureFocusVisible: m.ensureCursorView})
-	return output
+	switch overlayOp := m.op.(type) {
+	case *rebase.Operation:
+		overlayOp.ViewRect(dl, box)
+	case *duplicate.Operation:
+		overlayOp.ViewRect(dl, box)
+	case *revert.Operation:
+		overlayOp.ViewRect(dl, box)
+	case *squash.Operation:
+		overlayOp.ViewRect(dl, box)
+	}
+
+	// Reset the flag after ensuring cursor is visible
+	m.ensureCursorView = false
 }
 
 func (m *Model) load(revset string, selectedRevision string) tea.Cmd {
@@ -1033,14 +1066,20 @@ func (m *Model) selectRevision(revision string) int {
 	return idx
 }
 
-func (m *Model) search(startIndex int) int {
+func (m *Model) search(startIndex int, backward bool) int {
 	if m.quickSearch == "" {
 		return m.cursor
 	}
 
 	n := len(m.rows)
-	for i := startIndex; i < n+startIndex; i++ {
-		c := i % n
+	for i := range n {
+		var c int
+		if !backward {
+			c = (startIndex + i) % n
+		} else {
+			// calculate circular index going backwards
+			c = (startIndex - i + n) % n
+		}
 		row := &m.rows[c]
 		for _, line := range row.Lines {
 			for _, segment := range line.Segments {
@@ -1068,8 +1107,6 @@ func (m *Model) GetCommitIds() []string {
 func New(c *appContext.MainContext) *Model {
 	keymap := config.Current.GetKeyMap()
 	m := Model{
-		ViewNode:      common.NewViewNode(0, 0),
-		MouseAware:    common.NewMouseAware(),
 		context:       c,
 		keymap:        keymap,
 		rows:          nil,
@@ -1081,7 +1118,7 @@ func New(c *appContext.MainContext) *Model {
 		selectedStyle: common.DefaultPalette.Get("revisions selected"),
 		matchedStyle:  common.DefaultPalette.Get("revisions matched"),
 	}
-	m.renderer = newRevisionListRenderer(&m, m.ViewNode)
+	m.displayContextRenderer = NewDisplayContextRenderer(m.textStyle, m.dimmedStyle, m.selectedStyle, m.matchedStyle)
 	return &m
 }
 
