@@ -9,6 +9,9 @@ import (
 	"github.com/stretchr/testify/assert"
 )
 
+// In order to get a log output for testing, run the following command:
+// jj log -T "stringify('_PREFIX:' ++ separate('_PREFIX:', change_id.shortest(), commit_id.shortest(), divergent)) ++ ' ' ++ builtin_log_compact" --color always > test/testdata/your_test_case.log
+
 func TestParser_Parse(t *testing.T) {
 	file, _ := os.Open("testdata/output.log")
 	rows := parser.ParseRows(file)
@@ -23,6 +26,15 @@ func TestParser_Parse_WorkingCopyCommit(t *testing.T) {
 	assert.True(t, row.Commit.IsWorkingCopy)
 }
 
+func TestParser_Parse_DivergentLog(t *testing.T) {
+	file, _ := os.Open("testdata/divergent.log")
+	rows := parser.ParseRows(file)
+	assert.Len(t, rows, 3)
+	assert.True(t, strings.HasSuffix(rows[0].Commit.ChangeId, "/0"))
+	assert.True(t, strings.HasSuffix(rows[1].Commit.ChangeId, "/1"))
+	assert.Equal(t, "zo/2", rows[2].Commit.ChangeId)
+}
+
 // TODO: what does no-commit-id mean?
 // `testdata/no-commit-id.log` has commit id
 func TestParser_Parse_NoCommitId(t *testing.T) {
@@ -71,10 +83,10 @@ func TestParser_Parse_ConflictedLongIds(t *testing.T) {
 
 func TestParser_Parse_Disconnected(t *testing.T) {
 	var lb LogBuilder
-	lb.Write("*   short_id=abcde - short_id=xyrq - false id=abcde author=some@author id=xyrq")
+	lb.Write("*   _PREFIX:abcde_PREFIX:xyrq_PREFIX:false id=abcde author=some@author id=xyrq")
 	lb.Write("│   some documentation")
 	lb.Write("~\n")
-	lb.Write("*   short_id=abcde - short_id=xyrq - false id=abcde author=some@author id=xyrq")
+	lb.Write("*   _PREFIX:abcde_PREFIX:xyrq_PREFIX:false id=abcde author=some@author id=xyrq")
 	lb.Write("│   another commit")
 	lb.Write("~\n")
 	rows := parser.ParseRows(strings.NewReader(lb.String()))
@@ -84,7 +96,7 @@ func TestParser_Parse_Disconnected(t *testing.T) {
 
 func TestParser_Parse_Extend(t *testing.T) {
 	var lb LogBuilder
-	lb.Write("*   short_id=abcde - short_id=xyrq - false d=abcde author=some@author id=xyrq")
+	lb.Write("*   _PREFIX:abcde_PREFIX:xyrq_PREFIX:false id=abcde author=some@author id=xyrq")
 	lb.Write("│   some documentation")
 
 	rows := parser.ParseRows(strings.NewReader(lb.String()))
@@ -92,14 +104,14 @@ func TestParser_Parse_Extend(t *testing.T) {
 	row := rows[0]
 
 	extended := row.Extend()
-	assert.Len(t, extended.Segments, 3)
+	assert.Len(t, extended.Segments, 2)
 }
 
 func TestParser_Parse_WorkingCopy_1(t *testing.T) {
 	var lb LogBuilder
-	lb.Write("*   short_id=abcde - short_id=xyrq - false id=abcde author=some@author id=xyrq")
+	lb.Write("*   _PREFIX:abcde_PREFIX:xyrq_PREFIX:false id=abcde author=some@author id=xyrq")
 	lb.Write("│   some documentation")
-	lb.Write("@   short_id=kdys - id=12cd - false short_id=kdys author=some@author id=12cd")
+	lb.Write("@   _PREFIX:kdys_PREFIX:12cd_PREFIX:false short_id=kdys author=some@author id=12cd")
 	lb.Write("│   some documentation")
 
 	rows := parser.ParseRows(strings.NewReader(lb.String()))
@@ -164,3 +176,27 @@ func TestParser_DivergentChangeIDShort(t *testing.T) {
 	assert.Equal(t, "43bd", rows[1].Commit.CommitId)
 	assert.Equal(t, true, rows[1].Commit.IsConflicting())
 }
+
+func TestParser_ChangeIDCommitIDSameColor(t *testing.T) {
+	file, _ := os.Open("testdata/change-commit-same-color.log")
+	rows := parser.ParseRows(file)
+	assert.Len(t, rows, 3)
+	assert.Equal(t, "vvr", rows[0].Commit.ChangeId)
+	assert.Equal(t, "ae", rows[0].Commit.CommitId)
+	assert.Equal(t, false, rows[0].Commit.IsConflicting())
+	assert.Equal(t, "l", rows[1].Commit.ChangeId)
+	assert.Equal(t, "6e", rows[1].Commit.CommitId)
+	assert.Equal(t, false, rows[1].Commit.IsConflicting())
+	assert.Equal(t, "xv", rows[2].Commit.ChangeId)
+	assert.Equal(t, "fa", rows[2].Commit.CommitId)
+	assert.Equal(t, false, rows[2].Commit.IsConflicting())
+}
+
+func TestParser_Evolog(t *testing.T) {
+	file, _ := os.Open("testdata/evolog.log")
+	rows := parser.ParseRows(file)
+	assert.Len(t, rows, 2)
+	assert.Equal(t, "l", rows[0].Commit.ChangeId)
+	assert.Equal(t, "98", rows[0].Commit.CommitId)
+	assert.Equal(t, false, rows[0].Commit.IsConflicting())
+}
